; ====================================================================
; -----------------------------------------------------------------
; SH2
; -----------------------------------------------------------------

		include	"mars/sh2/shmap.i"
		include	"mars/sh2/shmap68k.i"
		org CS3
		
; -----------------------------------------------------------------
; Master CPU
; -----------------------------------------------------------------

		obj MasterStart
SH2_Master
		dc.l SH2_Entry			; Cold Start PC
		dc.l M_STACK			; Cold Start SP
		dc.l SH2_Entry			; Manual Reset PC
		dc.l M_STACK			; Manual Reset SP

		dc.l error0			; Illegal instruction
		dc.l $00000000			; reserved
		dc.l error0			; Invalid slot instruction
		dc.l $20100400			; reserved
		dc.l $20100420			; reserved
		dc.l error0			; CPU address error
		dc.l error0			; DMA address error
		dc.l error0			; NMI vector
		dc.l error0			; User break vector

		dcb.l 19,0			; reserved

		dcb.l 32,error0			; Trap vectors

 		dc.l main_irq			; Level 1 IRQ
		dc.l main_irq			; Level 2 & 3 IRQ's
		dc.l main_irq			; Level 4 & 5 IRQ's
		dc.l main_irq			; PWM interupt
		dc.l main_irq			; Command interupt
		dc.l main_irq			; H Blank interupt
		dc.l main_irq			; V Blank interupt
		dc.l main_irq			; Reset Button

; ====================================================================
; ---------------------------------------------------------------
; Entry point
; ---------------------------------------------------------------

SH2_Entry:
		mov.l	#_sysreg,r14
		ldc	r14,gbr

		mov.w	r0,@(vintclr,gbr)
		mov.w	r0,@(vintclr,gbr)
		mov.w	r0,@(hintclr,gbr)	;clear IRQ ACK regs
		mov.w	r0,@(hintclr,gbr)
		mov.w	r0,@(cmdintclr,gbr)
		mov.w	r0,@(cmdintclr,gbr)
		mov.w	r0,@(pwmintclr,gbr)
		mov.w	r0,@(pwmintclr,gbr)

		mov.l	#_FRT,r1		; Set Free Run Timer
		mov	#$00,r0
		mov.b	r0,@(_TIER,r1)		;
		mov	#$e2,r0
		mov.b	r0,@(_TOCR,r1)		;
		mov	#$00,r0
		mov.b	r0,@(_OCR_H,r1)		;
		mov	#$01,r0
		mov.b	r0,@(_OCR_L,r1)		;
		mov	#0,r0
		mov.b	r0,@(_TCR,r1)		;
		mov	#1,r0
		mov.b	r0,@(_TCSR,r1)		;
		mov	#$00,r0
		mov.b	r0,@(_FRC_L,r1)		;
		mov.b	r0,@(_FRC_H,r1)		;

		mov	#$f2,r0			; reset setup
		mov.b	r0,@(_TOCR,r1)		;
		mov	#$00,r0
		mov.b	r0,@(_OCR_H,r1)		;
		mov	#$01,r0
		mov.b	r0,@(_OCR_L,r1)		;
		mov	#$e2,r0
		mov.b	r0,@(_TOCR,r1)		;

@wait_md:
		mov.l	@(comm0,gbr),r0		; wait fo the genesis to finish booting
		cmp/eq	#0,r0
		bf	@wait_md
		nop

		mov.l	#"SLAV",r1
@wait_slave:
		mov.l	@(comm8,gbr),r0		; wait for the slave to finish booting
		cmp/eq	r1,r0
		bf	@wait_slave
		nop

; ====================================================================
; ---------------------------------------------------------------
; Hotstart
; ---------------------------------------------------------------

m_hotstart:
		mov.l	#_sysreg,r14
		ldc	r14,gbr

 		mov	#FM,r0
 		mov.b	r0,@(adapter,gbr)

		mov.l	#_sysreg,r14
		ldc	r14,gbr

		mov.l	#VIRQ_ON+CMDIRQ_ON+HIRQ_ON,r0
 		mov.b	r0,@(1,gbr)

		mov.l	#$20,r0
		ldc	r0,sr

; =================================================================

 		mov.l	#Master_GoToHere,r0
		jmp	@r0
 		nop
		lits

;-------------------------------------------------------;
;UNIFIED IRQ HANDLER
;-------------------------------------------------------;

main_irq
		mov.l	r0,@-r15
		mov.l	r1,@-r15
		sts.l	pr,@-r15		;save registers

		stc	sr,r0			;SR holds IRQ level
		shlr2	r0
		and	#$3C,r0
		mov.l	#inttable,r1
		add	r1,r0			;create index into IRQ table
		mov.l	@r0,r1			;fetch address
		jsr	@r1			;call IRQ service routine
		nop

		lds.l	@r15+,pr
		mov.l	@r15+,r1		;get registers back
		mov.l	@r15+,r0
		rte				;return from exception
		nop
		lits

;-------------------------------------------------------;
;IRQ TABLE
;-------------------------------------------------------;
	align	4
inttable
		dc.l invalid_irq
		dc.l invalid_irq
		dc.l invalid_irq
		dc.l invalid_irq
		dc.l invalid_irq
		dc.l invalid_irq
		dc.l pwm_irq
		dc.l pwm_irq
		dc.l cmd_irq
		dc.l cmd_irq
		dc.l h_irq
		dc.l h_irq
		dc.l v_irq
		dc.l v_irq
		dc.l vres_irq
		dc.l vres_irq

;-------------------------------------------------------;
;INVALID IRQ
;-------------------------------------------------------;
invalid_irq
		rts		;return back to mainline
		nop

;---------------------------------------------------------------*
;	VRES Interrupt
;---------------------------------------------------------------*

vres_irq:
		mov.l	#_sysreg,r0
		ldc	r0,gbr

		mov.w	r0,@(vresintclr,gbr)

		mov.l	#_FRT,r1		; System Reset
		mov.b	@(_TOCR,r1),r0		;
		or	#$01,r0		;
		mov.b	r0,@(_TOCR,r1)		;
vresloop:
		bra	vresloop
		nop

		lits

;---------------------------------------------------------------*
;	V Interrupt
;---------------------------------------------------------------*

v_irq:
	mov.l	r2,@-r15
	stc.l	gbr,@-r15	;save regs

	mov.l	#_sysreg,r0
	ldc	r0,gbr	;GBR = addr of sys regs

	mov.l	#$f0,r0
	ldc	r0,sr	;mask off all IRQS

	mov.l	#_FRT,r1
	mov.b	@(_TOCR,r1),r0
	xor	#$02,r0	;toggle FRT bit as required
	mov.b	r0,@(_TOCR,r1)


	mov.w	r0,@(vintclr,gbr)	; V interrupt clear
	nop
	nop
	nop		;delay 2 cycles as required
	nop

	mov.l	#TH+vcounter,r1
	mov.l	@r1,r0
	add	#1,r0	;do some work
	mov.l	r0,@r1

	ldc.l @r15+,gbr
	mov.l	@r15+,r2	;restore regs

	rts		;return to unified IRQ handler
	nop		;RTS!!

	align	4

vcounter		dc.l $00000000

	lits

;---------------------------------------------------------------*
;	H Interrupt
;---------------------------------------------------------------*

h_irq:

	stc.l	gbr,@-r15	;save regs

	mov.l	#_sysreg,r0
	ldc	r0,gbr	;GBR = addr of sys regs

	mov.l	#$f0,r0
	ldc	r0,sr	;mask off all IRQS

	mov.l	#_FRT,r1
	mov.b	@(_TOCR,r1),r0
	xor	#$02,r0	;toggle FRT bit as required
	mov.b	r0,@(_TOCR,r1)


	mov.w	r0,@(hintclr,gbr)	; H interrupt clear

	nop
	nop
	nop		;delay 2 cycles as required
	nop

	mov.l	#TH+hcounter,r1
	mov.l	@r1,r0
	add	#1,r0	;do some work
	mov.l	r0,@r1

	ldc.l @r15+,gbr	;restore regs

	rts		;return to unified IRQ handler
	nop		;RTS!!

	align	4
hcounter		dc.l $00000000

	lits

;---------------------------------------------------------------*
;	CMD Interrupt
;---------------------------------------------------------------*

cmd_irq:
	mov.l	r2,@-r15
	stc.l	gbr,@-r15	;save regs

	mov.l	#_sysreg,r0
	ldc	r0,gbr	;GBR = addr of sys regs

	mov.l	#$f0,r0
	ldc	r0,sr	;mask off all IRQS

	mov.l	#_FRT,r1
	mov.b	@(_TOCR,r1),r0
	xor	#$02,r0	;toggle FRT bit as required
	mov.b	r0,@(_TOCR,r1)


	mov.w	r0,@(cmdintclr,gbr)	; CMD interrupt clear

	nop
	nop
	nop		;delay 2 cycles as required
	nop

	mov.l	#TH+cmdcounter,r1
	mov.l	@r1,r0
	add	#1,r0	;do some work
	mov.l	r0,@r1

	ldc.l @r15+,gbr
	mov.l	@r15+,r2	;restore regs

	rts		;return to unified IRQ handler
	nop		;RTS!!

	align	4
cmdcounter		dc.l $00000000

	lits


;---------------------------------------------------------------*
;	PWM Interrupt
;---------------------------------------------------------------*

pwm_irq:
	stc.l	gbr,@-r15	;save regs

	mov.l	#_sysreg,r0
	ldc	r0,gbr	;GBR = addr of sys regs

	mov.l	#$f0,r0
	ldc	r0,sr	;mask off all IRQS

	mov.l	#_FRT,r1
	mov.b	@(_TOCR,r1),r0
	xor	#$02,r0	;toggle FRT bit as required
	mov.b	r0,@(_TOCR,r1)

	mov.w	r0,@(pwmintclr,gbr)	; PWM interrupt clear

	nop
	nop		;delay 2 cycles as required
	nop
	nop

	ldc.l @r15+,gbr	;restore regs

	rts		;return to unified IRQ handler
	nop		;RTS!!

	lits

;---------------------------------------------------------------*
;	ERROR
;---------------------------------------------------------------*

error0:
	bra	error0	;bad IRQ HALT
	nop

	lits

; =================================================================
; ----------------------------------------
; Subs
; ----------------------------------------

		align	4

;---------------------------------------------------------------
; Load RAW pal to MARS
;
; Input:
; r1 - File
; r2 - Start from
; r3 - Length
;---------------------------------------------------------------

		cnop 0,4
MarsVdp_Mode1_LoadPal:
		mov.l	#_palette,r8
		mov.l	#_vdpreg,r4

@wait		mov.b	@(vdpsts,r4),r0
		tst	#$20,r0
		bt	@wait
	
		mov.l	#256,r3
		mov.l	#0,r0
		mov.l	#0,r4
		
@PalLoop:
		mov.b	@r1+,r0			;RED
		and	#$FF,r0
 		shlr2	r0
		shlr 	r0
		mov	r0,r4

 		mov.b	@r1+,r0			;GREEN
		and	#$FF,r0
 		shlr2	r0
 		shlr 	r0
 		shll2	r0
 		shll2	r0
 		shll	r0
 		add	r0,r4
 
 		mov.b	@r1+,r0			;BLUE
		and	#$FF,r0
 		shlr2	r0
 		shlr 	r0
 		shll8	r0
 		shll2	r0
 		add	r0,r4
 		
 		mov.w	r4,@r8
 		add.l	#2,r8
 		
 		dt	r3
 		bf	@PalLoop
		
		rts
		nop
		cnop 0,4
		lits
		
;---------------------------------------------------------------
; Load art
;
; Input:
; r1 - Data
; r2 - Xpos
; r3 - Ypos
; r4 - Width
; r5 - Height
;---------------------------------------------------------------

		cnop 0,4
MarsVdp_Mode1_LoadArt:
		mov.l	#_framebuffer+$200,r8
		add	r2,r8

  		mov	r3,r9
  		add 	r7,r9
  		mov	r9,r0
  		cmp/eq	#0,r0
  		bt	@zero_y
@set_y:
		add	r4,r8
  		dt	r9
  		bf	@set_y
@zero_y:

; -------------------------
; Start
; -------------------------

@loop_Y:
		mov.l	r8,r10
		mov.l	r1,r11
		mov.l	r4,r12

@loop_X:
		mov.b	@r1+,r7		;1-Put
		mov.b	r7,@r8		;2-byte
 		add 	#1,r8		;3-to FrameBuffer
		
 		dt	r4
 		bf	@loop_X

		mov.l	r12,r4
		mov.l	r11,r1
		mov.l	r10,r8
		add	r4,r1
		add	r4,r8

		dt	r5
		bf	@loop_Y
		rts
		nop
		cnop 0,4
		lits
		
; ----------------------------------------------------
; Set line start table
; ----------------------------------------------------

Set_Line_Start:
		mov	#$100,r0
		mov	#_framebuffer,r1
		mov	#160,r2
		mov	#256,r3
@loop
		mov.w	r0,@r1
		add	#2,r1
		dt	r3
		bf/s	@loop
		add	r2,r0
		rts
		nop
		lits
		cnop 0,4

; ----------------------------------------------------
; Switch frame buffers
; ----------------------------------------------------

Switch_Frame_Buffers:
		mov	#_vdpreg,r1

		mov	#curr_fb,r2
		mov.b	@(framectl,r1),r0
		xor	#1,r0
		mov.b	r0,@(framectl,r1)
		mov.b	r0,@r2

		rts
		nop
		lits
		cnop 0,4
		
; ----------------------------------------------------
; Wait frame swap
;
; Make sure frame buffer has swapped
; ----------------------------------------------------

Wait_Frame_Swap:
		mov	#_vdpreg,r1
		mov	#curr_fb,r2
		mov.b	@r2,r0
		mov	r0,r2
@wait3:
		mov.b	@(framectl,r1),r0
		cmp/eq	r0,r2
		bf	@wait3
	
		rts
		nop
		lits
		cnop 0,4
		
; ----------------------------------------------------------------

		cnop 0,4
SetVideoMode
		mov	r1,r0
		mov.l	#_vdpreg,r1
		mov.b	r0,@(bitmapmd,r1)

		rts
		nop
		lits
		cnop 0,4
		
; =====================================================================
; -------------------------------------------
; Main
; -------------------------------------------

Master_GoToHere:
  		bsr	Switch_Frame_Buffers
  		nop
  		bsr	Wait_Frame_Swap
  		nop
  		
 		mov	#1,r1
 		bsr	SetVideoMode
  		nop
  		bsr	Set_Line_Start
  		nop
 
 		mov.l	#mars_picture,r1
 		mov	#320,r4
 		mov	#224,r5
 		mov	#0,r2				;X start
 		mov	#0,r3				;Y start
 		bsr	MarsVdp_Mode1_LoadArt
 		nop
 		mov.l	#mars_pal,r1
		bsr	MarsVdp_Mode1_LoadPal
 		nop
 	
  		bsr	Switch_Frame_Buffers
  		nop
  		bsr	Wait_Frame_Swap
  		nop
  		
@loop:
		nop
		bra	@loop
		nop
		lits
		cnop 0,4
		
; =====================================================================
; -------------------------------------------
; Sample picture
; -------------------------------------------

		cnop 0,4
mars_picture:
		incbin "mars/data/test.raw"
		cnop 0,4		
mars_pal:	incbin "mars/data/test.raw.pal"
		cnop 0,4		
	
; ====================================================================
		
		objend

; ====================================================================
; ---------------------------------------------------------------
; Slave CPU
; ---------------------------------------------------------------

		cnop 0,(SlaveStart-CS3)
		obj SlaveStart
SH2_Slave:
		dc.l @EntryPoint		; Cold Start PC
		dc.l S_STACK			; Cold Start SP
		dc.l @EntryPoint		; Manual Reset PC
		dc.l S_STACK			; Manual Reset SP

		dc.l error0			; Illegal instruction
		dc.l $00000000			; reserved
		dc.l error0			; Invalid slot instruction
		dc.l $20100400			; reserved
		dc.l $20100420			; reserved
		dc.l error0			; CPU address error
		dc.l error0			; DMA address error
		dc.l error0			; NMI vector
		dc.l error0			; User break vector

		dcb.l 19,0			; reserved

		dcb.l 32,error0			; Trap vectors

		dc.l main_irq_s			; Level 1 IRQ
		dc.l main_irq_s			; Level 2 & 3 IRQ's
		dc.l main_irq_s			; Level 4 & 5 IRQ's
		dc.l main_irq_s			; PWM interupt
		dc.l main_irq_s			; Command interupt
		dc.l main_irq_s			; H Blank interupt
		dc.l main_irq_s			; V Blank interupt
		dc.l main_irq_s			; Reset Button

; ====================================================================
; ---------------------------------------------------------------
; Entry point
; ---------------------------------------------------------------

@EntryPoint:
		mov.l	#_sysreg,r14
		ldc	r14,gbr

		mov.l	#_FRT,r1		; Set Free Run Timer
		mov	#$00,r0
		mov.b	r0,@(_TIER,r1)		;
		mov	#$e2,r0
		mov.b	r0,@(_TOCR,r1)		;
		mov	#$00,r0
		mov.b	r0,@(_OCR_H,r1)		;
		mov	#$01,r0
		mov.b	r0,@(_OCR_L,r1)		;
		mov	#0,r0
		mov.b	r0,@(_TCR,r1)		;
		mov	#1,r0
		mov.b	r0,@(_TCSR,r1)		;
		mov	#$00,r0
		mov.b	r0,@(_FRC_L,r1)		;
		mov.b	r0,@(_FRC_H,r1)		;
wait_md:
		mov.l	@(comm0,gbr),r0
		cmp/eq	#0,r0
		bf	wait_md
	
		mov.l	#"SLAV",r0
		mov.l	r0,@(comm8,gbr)
	
		mov.l	#_vdpreg,r14

; ====================================================================
; ---------------------------------------------------------------
; Hotstart
; ---------------------------------------------------------------

s_hotstart:
		mov.l	#S_STACK,r15

		mov.l	#_sysreg,r14
		ldc	r14,gbr
	
		mov.l	#PWMIRQ_ON,r0
		mov.b	r0,@(1,gbr)		;enable IRQS

		mov.l	#$20,r0
		ldc	r0,sr

		mov.w	r0,@(pwmintclr,gbr)
		mov.w	r0,@(pwmintclr,gbr)
		mov.w	r0,@(vintclr,gbr)
		mov.w	r0,@(vintclr,gbr)
		mov.w	r0,@(hintclr,gbr)	;clear IRQ ACK regs
		mov.w	r0,@(hintclr,gbr)
		mov.w	r0,@(cmdintclr,gbr)
		mov.w	r0,@(cmdintclr,gbr)

; =================================================================
 		
 		mov.l	#Slave_GoToHere,r0
		jmp	@r0
 		nop
		lits
		
; =====================================================================
;-------------------------------------------------------;
;UNIFIED IRQ HANDLER
;-------------------------------------------------------;

main_irq_s:
		mov.l	r0,@-r15
		mov.l	r1,@-r15
		mov.l	r2,@-r15

		stc	sr,r1
		mov.l	#$F0,r2
		ldc	r2,sr
		mov.l	#$FFFFFE10,r2
		xor	r0,r0
		mov.b	r0,@(_TOCR,r2)

		sts.l	pr,@-r15	;save registers
		mov.l	r1,r0
		shlr2	r0
		and	#$3C,r0
		mov.l	#@inttable,r1
		mov.l	@(r0,r1),r0
		jsr	@r0		;call IRQ service routine
		nop
		lds.l	@r15+,pr

		mov.l	@r15+,r2	;get registers back
		mov.l	@r15+,r1	;get registers back
		mov.l	@r15+,r0
	
		rte			;return from exception
		nop
		lits

;-------------------------------------------------------;
;IRQ TABLE
;-------------------------------------------------------;
		.align	4
@inttable
		dc.l @invalid_irq
		dc.l @invalid_irq
		dc.l @invalid_irq
		dc.l @invalid_irq
		dc.l @invalid_irq
		dc.l @invalid_irq
		dc.l @pwm_irq
		dc.l @pwm_irq
		dc.l @cmd_irq
		dc.l @cmd_irq
		dc.l @h_irq
		dc.l @h_irq
		dc.l @v_irq
		dc.l @v_irq
		dc.l @vres_irq
		dc.l @vres_irq

;-------------------------------------------------------;
;INVALID IRQ
;-------------------------------------------------------;

@invalid_irq
		rts
		nop

;---------------------------------------------------------------*
;	VRES Interrupt
;---------------------------------------------------------------*

@vres_irq:
		mov.l	#_sysreg,r0
		ldc	r0,gbr

		mov.w	r0,@(vresintclr,gbr)	; V interrupt clear

		mov.b	@(dreqctl,gbr),r0
		tst	#1,r0
		bf	@vresloop

		mov.l	#S_STACK-8,r15		; reset the stack
		mov.l	#s_hotstart,r0
		mov	r0,@r15
		mov.w	#$f0,r0
		mov	r0,@(4,r15)

		mov.l	#_DMAOPERATION,r1
		mov	#0,r0
		mov.l	r0,@r1

		mov.l	#_DMACHANNEL0,r1
		mov	#0,r0
		mov.l	r0,@r1
		mov.l	#%0100010011100000,r1
		mov.l	r0,@r1

		rte
		nop

@vresloop:
		bra	@vresloop
		nop
		lits

;---------------------------------------------------------------*
;	V Interrupt
;---------------------------------------------------------------*

@v_irq:
		mov.l	r2,@-r15
		stc.l	gbr,@-r15	;save regs

		mov.l	#_sysreg,r0
		ldc	r0,gbr	;GBR = addr of sys regs

		mov.w	r0,@(vintclr,gbr)	; V interrupt clear
		mov.l	#_FRT,r1
		mov.l	#$02,r0		;toggle FRT bit for future IRQs
		mov.b	r0,@(_TOCR,r1)	;as required
		mov.w	@(vintclr,gbr),r0	; V interrupt clear
		mov.b	@(_TOCR,r1),r0	;as required

		mov.l	#TH+@vcounter,r1
		mov.l	@r1,r0
		add	#1,r0	;do some work
		mov.l	r0,@r1

		ldc.l	@r15+,gbr
		mov.l	@r15+,r2	;restore regs

		rts		;return to universal handler
		nop		;RTS!

		.align	4
@vcounter	dc.l	$00000000
		lits

;---------------------------------------------------------------*
;	H Interrupt
;---------------------------------------------------------------*

@h_irq:
		stc.l	gbr,@-r15	;save regs

		mov.l	#_sysreg,r0
		ldc	r0,gbr	;GBR = addr of sys regs

		mov.w	r0,@(hintclr,gbr)	; V interrupt clear
		mov.l	#_FRT,r1
		mov.l	#$02,r0	;toggle FRT bit for future IRQs
		mov.b	r0,@(_TOCR,r1)	;as required
		mov.w	@(hintclr,gbr),r0	; V interrupt clear
		mov.b	@(_TOCR,r1),r0	;as required


		mov.l	#TH+@hcounter,r1
		mov.l	@r1,r0
		add	#1,r0	;do some work
		mov.l	r0,@r1

		ldc.l	@r15+,gbr	;restore regs

		rts		;return to universal handler
		nop                                    ;RTS!

		.align	4
@hcounter	dc.l	$00000000

		lits

;---------------------------------------------------------------*
;	CMD Interrupt
;---------------------------------------------------------------*

@cmd_irq:
		mov.l	r2,@-r15
		stc.l	gbr,@-r15	;save regs

		mov.l	#_sysreg,r0
		ldc	r0,gbr	;GBR = addr of sys regs

		mov.w	r0,@(cmdintclr,gbr)	; V interrupt clear
		mov.l	#_FRT,r1
		mov.l	#$02,r0	;toggle FRT bit for future IRQs
		mov.b	r0,@(_TOCR,r1)	;as required
		mov.w	@(cmdintclr,gbr),r0	; V interrupt clear
		mov.b	@(_TOCR,r1),r0	;as required

		mov.l	#TH+@cmdcounter,r1
		mov.l	@r1,r0
		add	#1,r0	;do some work
		mov.l	r0,@r1

		ldc.l	@r15+,gbr
		mov.l	@r15+,r2	;restore regs

		rts		;return to universal handler
		nop		;RTS!

		.align	4
@cmdcounter	dc.l	$00000000
		lits

;---------------------------------------------------------------*
;	PWM Interrupt
;---------------------------------------------------------------*
 
@pwm_irq
	stc.l	gbr,@-r15	;save regs

	mov.l	#_sysreg,r0
	ldc	r0,gbr	;GBR = addr of sys regs

	mov.l	#$f0,r0
	ldc	r0,sr	;mask off all IRQS

	mov.l	#_FRT,r1
	mov.b	@(_TOCR,r1),r0
	xor	#$02,r0	;toggle FRT bit as required
	mov.b	r0,@(_TOCR,r1)

	mov.w	r0,@(pwmintclr,gbr)	; PWM interrupt clear

	nop
	nop		;delay 2 cycles as required
	nop
	nop

	ldc.l @r15+,gbr	;restore regs

	rts		;return to unified IRQ handler
	nop		;RTS!!

	lits
		
; =================================================================
; -------------------------------------------
; Subs
; -------------------------------------------

		
; =====================================================================
; -------------------------------------------
; Main
; -------------------------------------------

Slave_GoToHere:
		nop
		nop
@loop:
		nop
		nop
		bra	@loop
		
; ====================================================================

		objend
		
; ====================================================================
; ---------------------------------------------------------------
; RAM
; ---------------------------------------------------------------

		cnop 0,4
		include	"mars/sh2/ram.asm"
		